unit DLLForm_Unit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, FortressCommonDefs, Menus;

const
  PluginName: string = 'FortUI build 1020';

  WM_STARTGAME = WM_USER+1;
  WM_GETTURN = WM_USER+2;
  WM_SHOWGAMEINFO= WM_USER+3;

type

  TFormFromDLL = class(TForm)
    ExitButton: TButton;
    AvailProjectsListBox: TListBox;
    StartGameButton: TButton;
    FirstPlayerGroupBox: TGroupBox;
    SecondPlayerGroupBox: TGroupBox;
    FirstPlayerInfoMemo: TMemo;
    SecondPlayerInfoMemo: TMemo;
    Label1: TLabel;
    ShowLegendButton: TButton;
    LegendMemo: TMemo;
    Label2: TLabel;
    ProjectsPopupMenu: TPopupMenu;
    N1: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    Standart1: TMenuItem;
    ProjectActionsGroupBox: TGroupBox;
    LoadStandartProjectListButton: TButton;
    LoadProjectListButton: TButton;
    SaveProjectListButton: TButton;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    NextTurnButton: TButton;
    DebugOffButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure ExitButtonClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure AvailProjectsListBoxClick(Sender: TObject);
    procedure StartGameButtonClick(Sender: TObject);
    procedure AvailProjectsListBoxDblClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ShowLegendButtonClick(Sender: TObject);
    procedure LegendMemoDblClick(Sender: TObject);
    procedure GameLogButtonClick(Sender: TObject);
    procedure N1Click(Sender: TObject);
    procedure N2Click(Sender: TObject);
    procedure Standart1Click(Sender: TObject);
    procedure LoadStandartProjectListButtonClick(Sender: TObject);
    procedure LoadProjectListButtonClick(Sender: TObject);
    procedure SaveProjectListButtonClick(Sender: TObject);
    procedure NextTurnButtonClick(Sender: TObject);
    procedure DebugOffButtonClick(Sender: TObject);
  private
    procedure MessageHandler1(var Msg:TMessage); message WM_STARTGAME;
    procedure MessageHandler2(var Msg:TMessage); message WM_GETTURN;
    procedure MessageHandler3(var Msg:TMessage); message WM_SHOWGAMEINFO;
  public
    Player:integer;
    SyncEventHandle:THandle;
    PlayerTurn: integer;
    GameInfo:PGame;
    AGI:PAdditionalGameInfo;
    AvailProjects: PAvailProjects;

    StartingGame: boolean;
    GettingTurn: boolean;
    ShowGame: boolean;
    ShowTurn: boolean;

    FirstPlayerName:string;
    SecondPlayerName:string;

    GameResMsg,HelpMsg:string;
    procedure ShowPlayersInfo;
    procedure ShowAvailProjects;
    procedure UpdateHorzScrollListBox;
    procedure UpdateGameLog;
    procedure SaveProjectsList(aFileName:string);
    procedure LoadProjectsList(aFileName:string);
  end;

var
  FormFromDLL: TFormFromDLL;

implementation

uses Math;

{$R *.dfm}

procedure TFormFromDLL.FormCreate(Sender: TObject);
begin
  StartingGame:=false;
  GettingTurn:=false;
  FirstPlayerName:='P1';
  SecondPlayerName:='P2';
end;

procedure TFormFromDLL.MessageHandler1(var Msg: TMessage);
var
  _i:integer;
begin
  Label1.Caption:='   (15 )';
  StartGameButton.Visible:=true;
  StartGameButton.Enabled:=false;
  DebugOffButton.Visible:=false;
  NextTurnButton.Visible:=false;

  ShowLegendButton.Visible:=true;
  StartingGame:=true;
  GettingTurn:=false;
  ProjectActionsGroupBox.Visible:=true;
  AvailProjectsListBox.Clear;
  AvailProjectsListBox.MultiSelect:=True;
  for _i:=1 to TotalProjectsCount do
   AvailProjectsListBox.Items.Add(ProjectsStrings[_i]);
  UpdateHorzScrollListBox;
  AvailProjectsListBox.PopupMenu:=ProjectsPopupMenu;
  N1.Enabled:=false;
  SaveProjectListButton.Enabled:=N1.Enabled;
  FirstPlayerInfoMemo.Clear;
  SecondPlayerInfoMemo.Clear;
  Caption:=PluginName+'.  ';
end;

procedure TFormFromDLL.MessageHandler2(var Msg: TMessage);
begin
  ShowLegendButton.Visible:=True;
  Label1.Caption:='  ( )';
  Label2.Caption:='  : '+IntToStr(AGI^.TurnCount)
                  +#13#10'  : '+IntToStr(AGI^.GameFlags and $FFFF);
  if AGI^.PreviousOpponentProject<>0 then
   Label2.Caption:=Label2.Caption+#13#10'   : '+ProjectsStrings[AGI^.PreviousOpponentProject];
  GettingTurn:=true;
  StartGameButton.Visible:=false;
  DebugOffButton.Visible:=false;
  NextTurnButton.Visible:=false;
    
  AvailProjectsListBox.MultiSelect:=false;
  ProjectActionsGroupBox.Visible:=false;
  ShowPlayersInfo;
  ShowAvailProjects;
  FirstPlayerGroupBox.Caption:=' ';
  SecondPlayerGroupBox.Caption:=' ';
  AvailProjectsListBox.PopupMenu:=nil;
  Caption:=PluginName+'.  . (  '+IntToStr(AGI^.GameFlags and $FFFF)+')';
end;

procedure TFormFromDLL.ExitButtonClick(Sender: TObject);
begin
  PlayerTurn:=-1;
  if ShowGame then ShowGame:=false;
  if ShowTurn then
   begin
    PlayerTurn:=2;
    ShowTurn:=false;
   end;
  SetEvent(SyncEventHandle);
end;

procedure TFormFromDLL.MessageHandler3(var Msg: TMessage);
var
  _FirstPlayerTurn, _SecondPlayerTurn:WORD;
begin
  StartGameButton.Visible:=false;
  ShowLegendButton.Visible:=false;
  if IsFlagPresented(AGI^.GameFlags,GF_DEBUGMODE) then
   ShowTurn:=true
     else
   ShowGame:=true;

  NextTurnButton.Visible:=ShowTurn;
  DebugOffButton.Visible:=ShowTurn; 
  FirstPlayerGroupBox.Caption:='  ('+FirstPlayerName+')';
  SecondPlayerGroupBox.Caption:='  ('+SecondPlayerName+')';
  Label1.Caption:='';
  Label2.Caption:='  : '+IntToStr(AGI^.TurnCount);
  AvailProjectsListBox.MultiSelect:=false;
  AvailProjectsListBox.Clear;
  ShowPlayersInfo;
  AvailProjectsListBox.PopupMenu:=nil;
  if ShowTurn then
   begin
    _FirstPlayerTurn:=AGI^.PreviousOpponentProject and $FFFF;
    _SecondPlayerTurn:=AGI^.PreviousOpponentProject shr 16;
    Caption:=PluginName+'.  ';
    AvailProjectsListBox.Items.Add('   - '+ FirstPlayerName);
    if _FirstPlayerTurn<>0 then
     AvailProjectsListBox.Items.Add(ProjectsStrings[_FirstPlayerTurn])
                           else
     AvailProjectsListBox.Items.Add('');
    AvailProjectsListBox.Items.Add('   - '+SecondPlayerName);
    if _SecondPlayerTurn<>0 then
     AvailProjectsListBox.Items.Add(ProjectsStrings[_SecondPlayerTurn])
                               else
     AvailProjectsListBox.Items.Add('');
   end else
   begin
    Caption:=PluginName+'.  ';
   end;
end;

procedure TFormFromDLL.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  ExitButtonClick(ExitButton);
  CanClose:=false;
end;

procedure TFormFromDLL.AvailProjectsListBoxClick(Sender: TObject);
begin
  if GettingTurn or ShowTurn then exit;
  Label2.Caption:='  : '+IntToStr(AvailProjectsListBox.SelCount)+' : '+IntToStr(MaxProjectsToPLayer);

  StartGameButton.Enabled:=AvailProjectsListBox.SelCount=MaxProjectsToPLayer;
  N1.Enabled:=AvailProjectsListBox.SelCount=MaxProjectsToPLayer;
  SaveProjectListButton.Enabled:=  N1.Enabled;
end;

procedure TFormFromDLL.StartGameButtonClick(Sender: TObject);
begin
  PlayerTurn:=1;
  StartingGame:=false;
  SetEvent(SyncEventHandle);
end;

procedure TFormFromDLL.ShowPlayersInfo;
var
  _FP,_SP:TPlayerInfo;
begin
  _FP:=GameInfo^[Player];
  _SP:=GameInfo^[GetOtherPlayer(Player)];

  FirstPlayerInfoMemo.Clear;
  FirstPlayerInfoMemo.Lines.Add(' : '+inttostr(_FP.Base));
  FirstPlayerInfoMemo.Lines.Add(' : '+inttostr(_FP.Shield));
  FirstPlayerInfoMemo.Lines.Add(' (+'+inttostr(_FP.Battery)+') : '+inttostr(_FP.Energy));
  FirstPlayerInfoMemo.Lines.Add(' (+'+inttostr(_FP.Mines)+') : '+inttostr(_FP.Metal));
  FirstPlayerInfoMemo.Lines.Add(' (+'+inttostr(_FP.Labs)+') : '+inttostr(_FP.Elements));

  SecondPlayerInfoMemo.Clear;
  SecondPlayerInfoMemo.Lines.Add(' : '+inttostr(_SP.Base));
  SecondPlayerInfoMemo.Lines.Add(' : '+inttostr(_SP.Shield));
  SecondPlayerInfoMemo.Lines.Add(' (+'+inttostr(_SP.Battery)+') : '+inttostr(_SP.Energy));
  SecondPlayerInfoMemo.Lines.Add(' (+'+inttostr(_SP.Mines)+') : '+inttostr(_SP.Metal));
  SecondPlayerInfoMemo.Lines.Add(' (+'+inttostr(_SP.Labs)+') : '+inttostr(_SP.Elements));
end;

procedure TFormFromDLL.ShowAvailProjects;
var
  _i:integer;
begin
  AvailProjectsListBox.Clear;
  for _i:=0 to MaxProjectsToPlayer-1 do
   if AvailProjects[_i]<>0 then
    AvailProjectsListBox.Items.Add(ProjectsStrings[AvailProjects^[_i]]);
  UpdateHorzScrollListBox;  
end;

procedure TFormFromDLL.AvailProjectsListBoxDblClick(Sender: TObject);
{var
  _SE:THandle;}
begin
  if GettingTurn then
   begin
    GettingTurn:=False;
    PlayerTurn:=AvailProjects^[AvailProjectsListBox.ItemIndex];
    SetEvent(SyncEventHandle);
    {_SE:=OpenEvent(EVENT_MODIFY_STATE,false,'SyncEvent');
    SetEvent(_SE);
    sleep(1);
    closehandle(_SE); }
   end;
end;



procedure TFormFromDLL.FormActivate(Sender: TObject);
var
  MainWindow:THandle;
begin
  ShowWindow(FormFromDLL.Handle,SW_HIDE);

  MainWindow:=FindWindow('TFortressMainForm',nil);
  SetForegroundWindow(MainWindow);
end;

procedure TFormFromDLL.UpdateHorzScrollListBox;
var  
  _i, MaxWidth: integer;
begin
  MaxWidth := 0;
  for _i := 0 to AvailProjectsListBox.Items.Count - 1 do
  if MaxWidth < AvailProjectsListBox.Canvas.TextWidth(AvailProjectsListBox.Items.Strings[_i]) then
    MaxWidth := AvailProjectsListBox.Canvas.TextWidth(AvailProjectsListBox.Items.Strings[_i]);
  SendMessage(AvailProjectsListBox.Handle, LB_SETHORIZONTALEXTENT, MaxWidth+2, 0);

end;

procedure TFormFromDLL.ShowLegendButtonClick(Sender: TObject);
begin
  LegendMemo.Visible:=true;
end;

procedure TFormFromDLL.LegendMemoDblClick(Sender: TObject);
begin
  LegendMemo.Visible:=false;
end;

procedure TFormFromDLL.GameLogButtonClick(Sender: TObject);
begin
  //
end;

procedure TFormFromDLL.UpdateGameLog;
var
  _i:integer;
begin

end;

procedure TFormFromDLL.SaveProjectsList(aFileName: string);
var
  _str:TStringList;
  _i:integer;
begin
  _str:=TStringList.Create;
  for _i:=0 to AvailProjectsListBox.Count-1 do
   if AvailProjectsListBox.Selected[_i] then
    _str.Add(IntToStr(_i));

  _str.SaveToFile(aFileName);
  _str.Free;
end;

procedure TFormFromDLL.LoadProjectsList(aFileName: string);
var
  _str:TStringList;
  _i:integer;
begin
  _str:=TStringList.Create;
  if not FileExists(aFileName) then exit;
  _str.LoadFromFile(aFileName);

  if _str.Count<>MaxProjectsToPLayer then exit;
  for _i:=0 to AvailProjectsListBox.Count-1 do
   AvailProjectsListBox.Selected[_i]:=false;

  try
   for _i:=0 to _str.Count-1 do
    AvailProjectsListBox.Selected[strtoint(_str[_i])]:=true;
  except
   MessageDlg('   !', mtError, [mbOK],0);
   for _i:=0 to AvailProjectsListBox.Count-1 do
    AvailProjectsListBox.Selected[_i]:=false;
  end;

  _str.Free;
  AvailProjectsListBoxClick(AvailProjectsListBox);
end;

procedure TFormFromDLL.N1Click(Sender: TObject);
var
  _PLN:string;
begin
  _PLN:=InputBox(' ','     ','save1');
  SaveProjectsList(_PLN+'.txt');
end;

procedure TFormFromDLL.N2Click(Sender: TObject);
var
  _PLN:string;
begin
  _PLN:=InputBox(' ','   ','save1');
  LoadProjectsList(_PLN+'.txt'); 
end;

procedure TFormFromDLL.Standart1Click(Sender: TObject);
begin
  LoadProjectsList('standart.txt');
end;

procedure TFormFromDLL.LoadStandartProjectListButtonClick(Sender: TObject);
begin
  LoadProjectsList('standart.txt');
end;

procedure TFormFromDLL.LoadProjectListButtonClick(Sender: TObject);
begin
  N2Click(Sender);
end;

procedure TFormFromDLL.SaveProjectListButtonClick(Sender: TObject);
begin
  N1Click(Sender);
end;

procedure TFormFromDLL.NextTurnButtonClick(Sender: TObject);
begin
  if not ShowTurn then exit;
  PlayerTurn:=0;
  ShowTurn:=false;
  SetEvent(SyncEventHandle);
end;

procedure TFormFromDLL.DebugOffButtonClick(Sender: TObject);
begin
  if not ShowTurn then exit;
  PlayerTurn:=1;
  ShowTurn:=false;
  SetEvent(SyncEventHandle);
end;

end.
