unit FortressCore;

interface

uses Sysutils, FortressCommonDefs,  classes, windows;

 
type
  TPlayerStat = record
    TotalDamage:DWORD;
    TotalEnergyCollected:DWORD;
    TotalMetalCollected:DWORD;
    TotalElementsCollected:DWORD;
    Time:Double;
    BaseAvg:Double;
    ShieldAvg:Double;
    BatteriesAvg:Double;
    MinesAvg:Double;
    LabsAvg:Double;
    ProjectsStat: array[1..TotalProjectsCount] of integer;
   end;
   
  TFullPlayerInfo = record
    Base:integer;    // 
    Shield:integer;  // 
    Battery:integer; // +   
    Mines:integer;   // +   
    Labs:integer;    // +   

    Energy:integer;  //   
    Metal:integer;   //   
    Elements:integer;//   

    Projects: array[0..MaxProjectsToPlayer-1] of integer;
    Statistic:TPlayerStat;
   end;
  PFullPlayerInfo = ^ TFullPlayerInfo; 

  TFullGameInfo = array[0..1] of TFullPlayerInfo;
  PFullGameInfo = ^ TFullGameInfo;

  TProjectHandler = function(aFullGameInfo:PFullGameInfo;const CurrentPlayer:integer):integer;

function FullGameInfoToGame(aFGI:PFullGameInfo; GameFlags:DWORD; PlayerNumber:integer):TGame;
function DoProject(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer; aProject: integer):boolean;
function CheckAvailProjects(aAvailProjects: PAvailProjects):boolean;
function GetAvailProjects(aPlayerInfo:PFullPlayerInfo;aAvailProjects:PAvailProjects):integer;
function IsPlayerLose(aPlayerInfo:PFullPlayerInfo):boolean;
function CheckBotName(aName:PChar):boolean;
procedure SetOppName(AGI:PAdditionalGameInfo; aName:string);
procedure InitFullGameInfo(aFullGameInfo:PFullGameInfo);
procedure CalcAvgs(var aPlayerStat:TPlayerStat; CountValue:integer);

procedure AddStat(var A:TPlayerStat;B:TPlayerStat);

implementation

function CheckBotName(aName:PChar):boolean;
var
  _i,_l:integer;
begin
  Result:=false;
  _l:=length(aName);
  if _l>BotNameMaxLength then exit;
  for _i:=0 to _l-1 do
   if not (aName[_i] in ['a'..'z','A'..'Z','0'..'9','_','-','.',' ','+','(',')']) then exit;
  Result:=true;
end;

procedure SetOppName(AGI:PAdditionalGameInfo; aName:string);
var
  _i,_l:integer;
begin
  _l:=length(aName);
  ZeroMemory(@AGI.OpponentName,BotNameMaxLength+1);
  for _i:=1 to _l do
   AGI^.OpponentName[_i-1]:=aName[_i];
end;

procedure AddStat(var A:TPlayerStat; B:TPlayerStat);
var
  _i:integer;
begin
  A.TotalDamage:=A.TotalDamage+B.TotalDamage;
  A.TotalEnergyCollected:=A.TotalEnergyCollected+B.TotalEnergyCollected;
  A.TotalMetalCollected:=A.TotalMetalCollected+B.TotalMetalCollected;
  A.TotalElementsCollected:=A.TotalElementsCollected+B.TotalElementsCollected;
  A.Time:=A.Time+B.Time;
  A.BaseAvg:=A.BaseAvg+B.BaseAvg;
  A.ShieldAvg:=A.ShieldAvg+B.ShieldAvg;
  A.BatteriesAvg:=A.BatteriesAvg+B.BatteriesAvg;
  A.MinesAvg:=A.MinesAvg+B.MinesAvg;
  A.LabsAvg:=A.LabsAvg+B.LabsAvg;

  for _i:=1 to TotalProjectsCount do
   A.ProjectsStat[_i]:=A.ProjectsStat[_i]+B.ProjectsStat[_i];
end;

procedure CalcAvgs(var aPlayerStat:TPlayerStat; CountValue:integer);
begin
  if CountValue<>0 then
   begin
    aPlayerStat.BaseAvg:=aPlayerStat.BaseAvg/CountValue;
    aPlayerStat.ShieldAvg:=aPlayerStat.ShieldAvg/CountValue;
    aPlayerStat.BatteriesAvg:=aPlayerStat.BatteriesAvg/CountValue;
    aPlayerStat.MinesAvg:=aPlayerStat.MinesAvg/CountValue;
    aPlayerStat.LabsAvg:=aPlayerStat.LabsAvg/CountValue;
   end;
end;

function GetOtherPlayer(aPlayer:integer):integer;
begin
  Result:=0;
  if aPlayer=0 then Result:=1;
end;

function IsFlagPresented(value:DWORD;aFlag:DWORD):boolean;
begin
  Result:= (value and aFlag) = aFlag;
end;

{
function IsProjectExistForPlayer():boolean;
begin
end;
}
procedure InitFullGameInfo(aFullGameInfo:PFullGameInfo);
begin
  ZeroMemory(aFullGameInfo,sizeof(TFullGameInfo));
  aFullGameInfo^[0].Statistic.BaseAvg:=0.0;
  aFullGameInfo^[1].Statistic.BaseAvg:=0.0;
end;

function FullGameInfoToGame(aFGI:PFullGameInfo; GameFlags:DWORD; PlayerNumber:integer):TGame;
var
  _OthPlr:integer;
begin
  if IsFlagPresented(GameFlags,GF_EXTENDED) then
   begin
    CopyMemory(@Result[PlayerNumber],@aFGI^[PlayerNumber],sizeof(TPlayerInfo));
    _OthPlr:=GetOtherPlayer(PlayerNumber);
    if aFGI^[_OthPlr].Shield>BlockShieldValue then
     begin
      Result[_OthPlr].Base:=0;
      Result[_OthPlr].Shield:=aFGI^[_OthPlr].Shield;
      Result[_OthPlr].Battery:=0;
      Result[_OthPlr].Mines:=0;
      Result[_OthPlr].Labs:=0;
      Result[_OthPlr].Energy:=-1;
      Result[_OthPlr].Metal:=-1;
      Result[_OthPlr].Elements:=-1;
     end                                       else
     CopyMemory(@Result[_OthPlr],@aFGI^[_OthPlr],sizeof(TPlayerInfo));
   end                                      else
   begin
    CopyMemory(@Result[0],@aFGI^[0],sizeof(TPlayerInfo));
    CopyMemory(@Result[1],@aFGI^[1],sizeof(TPlayerInfo));
   end;
end;

function IsPlayerLose(aPlayerInfo:PFullPlayerInfo):boolean;
begin
  Result:=aPlayerInfo^.Base<=0;
end;

function GetAvailProjects(aPlayerInfo:PFullPlayerInfo;aAvailProjects:PAvailProjects):integer;
var
  _i:integer;
begin
  ZeroMemory(aAvailProjects,sizeof(TAvailProjects));
  Result:=0;
  for _i:=0 to MaxProjectsToPlayer-1 do
   if aPlayerInfo^.Projects[_i]<>0 then
    if (ProjectsData[aPlayerInfo^.Projects[_i],1]<=aPlayerInfo^.Energy) and
       (ProjectsData[aPlayerInfo^.Projects[_i],2]<=aPlayerInfo^.Metal) and
       (ProjectsData[aPlayerInfo^.Projects[_i],3]<=aPlayerInfo^.Elements) then
     begin
      aAvailProjects^[Result]:=aPlayerInfo^.Projects[_i];
      inc(Result);
     end;
end;

function Attack1(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 5;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[1,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[1,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[1,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;        
   end;
   
end;

function Attack2(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 5;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[2,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[2,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[2,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;        
   end;   
end;

function Attack3(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 5;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[3,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[3,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[3,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;        
   end;   
end;

function Attack4(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 7;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[4,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[4,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[4,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Attack5(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 8;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[5,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[5,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[5,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Attack6(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 8;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[6,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[6,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[6,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;        
   end;   
end;

function Attack7(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 10;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[7,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[7,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[7,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Attack8(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 10;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[8,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[8,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[8,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Attack9(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 10;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[9,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[9,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[9,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Attack2_1(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 14;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[10,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[10,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[10,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;        
   end;   
end;

function Attack2_2(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 15;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[11,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[11,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[11,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Attack2_3(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 25;
  _SELF_DAMAGE = 10;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[12,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[12,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[12,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;        
   end;
      //SELF DAMAGE
  if aFullGameInfo^[CurrentPlayer].Shield>=_SELF_DAMAGE then
   aFullGameInfo^[CurrentPlayer].Shield:=aFullGameInfo^[CurrentPlayer].Shield-_SELF_DAMAGE
                                             else
   begin
    aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base-(_SELF_DAMAGE-aFullGameInfo^[CurrentPlayer].Shield);
    aFullGameInfo^[CurrentPlayer].Shield:=0;
   end;
end;

function Attack2_4(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 17;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[13,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[13,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[13,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Attack2_5(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 18;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[14,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[14,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[14,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Repair1(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[15,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[15,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[15,3];

  aFullGameInfo^[CurrentPlayer].Shield:=aFullGameInfo^[CurrentPlayer].Shield+7;
end;

function Repair2(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[16,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[16,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[16,3];

  aFullGameInfo^[CurrentPlayer].Shield:=aFullGameInfo^[CurrentPlayer].Shield+11;
end;

function Repair3(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[17,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[17,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[17,3];

  aFullGameInfo^[CurrentPlayer].Shield:=aFullGameInfo^[CurrentPlayer].Shield+16;
end;

function Repair4(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[18,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[18,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[18,3];

  aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base+9;
end;

function Repair5(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_REPAIR = 6;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[19,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[19,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[19,3];

  if aFullGameInfo^[CurrentPlayer].Shield>=_MAX_REPAIR then
   begin
    aFullGameInfo^[CurrentPlayer].Shield:=aFullGameInfo^[CurrentPlayer].Shield-_MAX_REPAIR;
    aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base+_MAX_REPAIR*2;
   end                                                 else
   begin
    aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base+aFullGameInfo^[CurrentPlayer].Shield*2;
    aFullGameInfo^[CurrentPlayer].Shield:=0;
   end;
end;

function Repair6(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[20,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[20,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[20,3];

  aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base+12;
end;

function Repair7(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[21,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[21,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[21,3];

  aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base+14;
end;

function Repair8(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[22,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[22,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[22,3];

  aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base+14;
  aFullGameInfo^[CurrentPlayer].Shield:=aFullGameInfo^[CurrentPlayer].Shield+15;
end;

function SpecAttack1(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[23,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[23,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[23,3];

  aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-3;
end;

function SpecAttack2(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[24,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[24,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[24,3];

  aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-7;
end;

function SpecAttack3(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[25,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[25,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[25,3];

  aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-7;
end;

function SpecAttack4(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX = 7;
var
  _damage:integer;
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[26,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[26,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[26,3];

  if aFullGameInfo^[CurrentPlayer].Shield>=_MAX then
   begin
    aFullGameInfo^[CurrentPlayer].Shield:=aFullGameInfo^[CurrentPlayer].Shield-_MAX;
    _damage:=1+2*_MAX;
   end                                       else
   begin
    _damage:=1+aFullGameInfo^[CurrentPlayer].Shield*2;
    aFullGameInfo^[CurrentPlayer].Shield:=0;
   end;

  if aFullGameInfo^[_OthPlr].Shield>=_damage then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_damage-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function SpecAttack5(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[27,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[27,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[27,3];

  if aFullGameInfo^[_OthPlr].Shield=0 then
   aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-11;
end;


function SuperAttack1(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _SDAMAGE = 10;
  _BDAMAGE = 10;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[28,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[28,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[28,3];

  aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_SDAMAGE;
  if aFullGameInfo^[_OthPlr].Shield<0 then aFullGameInfo^[_OthPlr].Shield:=0;
  aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-_BDAMAGE;
end;

function SuperAttack2(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _SDAMAGE = 15;
  _BDAMAGE = 20;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[29,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[29,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[29,3];

  aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_SDAMAGE;
  if aFullGameInfo^[_OthPlr].Shield<0 then aFullGameInfo^[_OthPlr].Shield:=0;
  aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-_BDAMAGE;
end;

function SuperAttack3(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 49;
  _SELF_DAMAGE = 20;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[30,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[30,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[30,3];


  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;

  if aFullGameInfo^[CurrentPlayer].Shield>=_SELF_DAMAGE then
   aFullGameInfo^[CurrentPlayer].Shield:=aFullGameInfo^[CurrentPlayer].Shield-_SELF_DAMAGE
                                             else
   begin
    aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base-(_SELF_DAMAGE-aFullGameInfo^[CurrentPlayer].Shield);
    aFullGameInfo^[CurrentPlayer].Shield:=0;
   end;

end;

function SuperAttack4(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 49;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[31,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[31,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[31,3];


  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function SuperAttack5(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 25;
  _BDAMAGE = 5;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[32,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[32,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[32,3];


  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
  aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-_BDAMAGE;
end;

function Development1(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[33,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[33,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[33,3];

  aFullGameInfo^[CurrentPlayer].Battery:=aFullGameInfo^[CurrentPlayer].Battery+1;
end;

function Development2(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[34,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[34,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[34,3];

  aFullGameInfo^[CurrentPlayer].Mines:=aFullGameInfo^[CurrentPlayer].Mines+1;
end;

function Development3(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[35,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[35,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[35,3];

  aFullGameInfo^[CurrentPlayer].Labs:=aFullGameInfo^[CurrentPlayer].Labs+1;
end;

function Development4(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[36,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[36,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[36,3];

  aFullGameInfo^[CurrentPlayer].Battery:=aFullGameInfo^[CurrentPlayer].Battery+1;
  aFullGameInfo^[CurrentPlayer].Mines:=aFullGameInfo^[CurrentPlayer].Mines+1;
end;

function Development5(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[37,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[37,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[37,3];

  aFullGameInfo^[CurrentPlayer].Labs:=aFullGameInfo^[CurrentPlayer].Labs+1;
  aFullGameInfo^[CurrentPlayer].Base:=aFullGameInfo^[CurrentPlayer].Base+7;
end;

function Development6(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_CONV = 18;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[38,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[38,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[38,3];

  if aFullGameInfo^[CurrentPlayer].Metal>=_MAX_CONV then
   begin
    aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-_MAX_CONV;
    aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+2*_MAX_CONV;
   end                                                 else
   begin
    aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+2*aFullGameInfo^[CurrentPlayer].Metal;
    aFullGameInfo^[CurrentPlayer].Metal:=0;
   end;
end;

function Development7(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_CONV = 15;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[39,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[39,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[39,3];

  if aFullGameInfo^[CurrentPlayer].Elements>=_MAX_CONV then
   begin
    aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-_MAX_CONV;
    aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal+_MAX_CONV;
   end                                                 else
   begin
    aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal+aFullGameInfo^[CurrentPlayer].Elements;
    aFullGameInfo^[CurrentPlayer].Elements:=0;
   end;
end;

function Development8(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_CONV = 18;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[40,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[40,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[40,3];

  if aFullGameInfo^[CurrentPlayer].Metal>=_MAX_CONV then
   begin
    aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-_MAX_CONV;
    aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+_MAX_CONV;
   end                                                 else
   begin
    aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+aFullGameInfo^[CurrentPlayer].Metal;
    aFullGameInfo^[CurrentPlayer].Metal:=0;
   end;
end;

function Espionage1(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_STEAL = 5;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[41,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[41,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[41,3];

  if aFullGameInfo^[_OthPlr].Metal>=_MAX_STEAL then
   begin
    aFullGameInfo^[_OthPlr].Metal:=aFullGameInfo^[_OthPlr].Metal-_MAX_STEAL;
    aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal+_MAX_STEAL;
   end                                         else
   begin
    aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal+aFullGameInfo^[_OthPlr].Metal;
    aFullGameInfo^[_OthPlr].Metal:=0;
   end;
end;

function Espionage2(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_STEAL = 5;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[42,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[42,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[42,3];

  if aFullGameInfo^[_OthPlr].Elements>=_MAX_STEAL then
   begin
    aFullGameInfo^[_OthPlr].Elements:=aFullGameInfo^[_OthPlr].Elements-_MAX_STEAL;
    aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+_MAX_STEAL;
   end                                         else
   begin
    aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+aFullGameInfo^[_OthPlr].Elements;
    aFullGameInfo^[_OthPlr].Elements:=0;
   end;
end;

function Espionage3(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_SHIELD = 7;
  _MAX_ENERGY = 5;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[43,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[43,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[43,3];

  if aFullGameInfo^[_OthPlr].Shield>=_MAX_SHIELD then
   aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_MAX_SHIELD
                                                 else
   begin
    aFullGameInfo^[_OthPlr].Shield:=0;
    if aFullGameInfo^[_OthPlr].Energy>=_MAX_ENERGY then
     begin
      aFullGameInfo^[_OthPlr].Energy:=aFullGameInfo^[_OthPlr].Energy-_MAX_ENERGY;
      aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy+_MAX_ENERGY;
     end                                           else
     begin
      aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy+aFullGameInfo^[_OthPlr].Energy;
      aFullGameInfo^[_OthPlr].Energy:=0;
     end;
   end;
end;

function Espionage4(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MIN_MINES = 2;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[44,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[44,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[44,3];

  if aFullGameInfo^[_OthPlr].Mines>_MIN_MINES then
   aFullGameInfo^[_OthPlr].Mines:=aFullGameInfo^[_OthPlr].Mines-1;
end;  

function Espionage5(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MIN_BATTERIES = 3;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[45,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[45,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[45,3];

  if aFullGameInfo^[_OthPlr].Battery>_MIN_BATTERIES then
   aFullGameInfo^[_OthPlr].Battery:=aFullGameInfo^[_OthPlr].Battery-1;
end;

function Espionage6(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _DAMAGE = 15;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[46,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[46,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[46,3];

  if aFullGameInfo^[_OthPlr].Shield>=_DAMAGE then
   begin
    aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_DAMAGE;
    aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy+_DAMAGE;
   end
                                             else
   begin
    aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-(_DAMAGE-aFullGameInfo^[_OthPlr].Shield);
    aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy+aFullGameInfo^[_OthPlr].Shield;
    aFullGameInfo^[_OthPlr].Shield:=0;
   end;
end;

function Espionage7(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_SHIELD = 15;
  _MAX_STEAL = 20;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[47,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[47,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[47,3];

  if aFullGameInfo^[_OthPlr].Shield>=_MAX_SHIELD then
   begin
    aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_MAX_SHIELD;
    if  aFullGameInfo^[_OthPlr].Metal>= _MAX_STEAL then
     aFullGameInfo^[_OthPlr].Metal:= aFullGameInfo^[_OthPlr].Metal-_MAX_STEAL
                                                   else
     aFullGameInfo^[_OthPlr].Metal:=0;
   end
                                             else
   begin
    aFullGameInfo^[_OthPlr].Shield:=0;
    if aFullGameInfo^[_OthPlr].Metal>= _MAX_STEAL then
     begin
      aFullGameInfo^[_OthPlr].Metal:=aFullGameInfo^[_OthPlr].Metal-_MAX_STEAL;
      aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal+_MAX_STEAL;
     end                                          else
     begin
      aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal+ aFullGameInfo^[_OthPlr].Metal;
      aFullGameInfo^[_OthPlr].Metal:=0;
     end;
   end;
end;

function Espionage8(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_SHIELD = 15;
  _MAX_STEAL = 22;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[48,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[48,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[48,3];

  aFullGameInfo^[_OthPlr].Base:=aFullGameInfo^[_OthPlr].Base-_MAX_SHIELD;
  if aFullGameInfo^[_OthPlr].Shield>=_MAX_SHIELD then
   begin
    if aFullGameInfo^[_OthPlr].Energy>= _MAX_STEAL then
     aFullGameInfo^[_OthPlr].Energy:= aFullGameInfo^[_OthPlr].Energy-_MAX_STEAL
                                                    else
     aFullGameInfo^[_OthPlr].Energy:=0;
   end
                                             else
   begin
    if aFullGameInfo^[_OthPlr].Energy>= _MAX_STEAL then
     begin
      aFullGameInfo^[_OthPlr].Energy:=aFullGameInfo^[_OthPlr].Energy-_MAX_STEAL;
      aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy+_MAX_STEAL;
     end                                          else
     begin
      aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy+ aFullGameInfo^[_OthPlr].Energy;
      aFullGameInfo^[_OthPlr].Energy:=0;
     end;
   end;
end;

function Espionage9(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
const
  _MAX_SHIELD = 10;
  _MAX_STEAL = 20;
var
  _OthPlr:integer;
begin
  _OthPlr:=GetOtherPlayer(CurrentPlayer);

  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy-ProjectsData[49,1];
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal-ProjectsData[49,2];
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements-ProjectsData[49,3];

  if aFullGameInfo^[_OthPlr].Shield>=_MAX_SHIELD then
   begin
    aFullGameInfo^[_OthPlr].Shield:=aFullGameInfo^[_OthPlr].Shield-_MAX_SHIELD;
   end
                                             else
   begin
    aFullGameInfo^[_OthPlr].Shield:=0;
    if aFullGameInfo^[_OthPlr].Elements>= _MAX_STEAL then
     begin
      aFullGameInfo^[_OthPlr].Elements:=aFullGameInfo^[_OthPlr].Elements-_MAX_STEAL;
      aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+_MAX_STEAL;
     end                                          else
     begin
      aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+ aFullGameInfo^[_OthPlr].Elements;
      aFullGameInfo^[_OthPlr].Elements:=0;
     end;
   end;
end;

function SleepTurn(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer):integer;
begin
  aFullGameInfo^[CurrentPlayer].Energy:=aFullGameInfo^[CurrentPlayer].Energy+1;
  aFullGameInfo^[CurrentPlayer].Metal:=aFullGameInfo^[CurrentPlayer].Metal+1;
  aFullGameInfo^[CurrentPlayer].Elements:=aFullGameInfo^[CurrentPlayer].Elements+1;
end;

const
  ProjectsHandlers:array[1..TotalProjectsCount] of TProjectHandler =
  (
   Attack1,
   Attack2,
   Attack3,
   Attack4,
   Attack5,
   Attack6,
   Attack7,
   Attack8,
   Attack9,
   Attack2_1,
   Attack2_2,
   Attack2_3,
   Attack2_4,
   Attack2_5,
   Repair1,
   Repair2,
   Repair3,
   Repair4,
   Repair5,
   Repair6,
   Repair7,
   Repair8,
   SpecAttack1,
   SpecAttack2,
   SpecAttack3,
   SpecAttack4,
   SpecAttack5,
   SuperAttack1,
   SuperAttack2,
   SuperAttack3,
   SuperAttack4,
   SuperAttack5,
   
   Development1,
   Development2,
   Development3,
   Development4,
   Development5,
   Development6,
   Development7,
   Development8,
   Espionage1,
   Espionage2,
   Espionage3,
   Espionage4,
   Espionage5,
   Espionage6,
   Espionage7,
   Espionage8,
   Espionage9,
   SleepTurn
   );

function IsProjectAvail(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer; aProject: integer):boolean;
var
  _i:integer;
begin
  Result:=false;
  if (aProject<1) or (aProject>TotalProjectsCount) then exit;
  for _i:=0 to MaxProjectsToPLayer-1 do
   if aFullGameInfo^[CurrentPlayer].Projects[_i]=aProject then
    begin
     Result:=true;
     break;
    end;

  if not Result then exit;

  if (ProjectsData[aProject,1]>aFullGameInfo^[CurrentPlayer].Energy) or
     (ProjectsData[aProject,2]>aFullGameInfo^[CurrentPlayer].Metal) or
     (ProjectsData[aProject,3]>aFullGameInfo^[CurrentPlayer].Elements) then
   Result:=false;
end;

function IsProjectExists(aProject:integer):boolean;
begin
  Result:=ProjectsStrings[aProject]<>'';
end;

function CheckAvailProjects(aAvailProjects: PAvailProjects):boolean;
var
  _i,_j:integer;
begin
  Result:=true;
  for _i:=0 to MaxProjectsToPlayer-1 do
   begin
    if (aAvailProjects^[_i]<1) or (aAvailProjects^[_i]>TotalProjectsCount) then
     begin
      Result:=false;
      exit;
     end;
    if not IsProjectExists(aAvailProjects^[_i]) then
     begin
      Result:=false;
      exit;
     end;
   end;
  for _i:=0 to MaxProjectsToPlayer-2 do
   for _j:=_i+1 to MaxProjectsToPlayer-1 do
    if aAvailProjects^[_i]=aAvailProjects^[_j] then
     begin
      Result:=false;
      exit;
     end;
end;

procedure CheckLabLimit(aFullGameInfo:PFullGameInfo; const aPlayer:integer);
var
  _LabLimit:integer;
begin
  if aFullGameInfo^[aPlayer].Base>UnlimitLabsBaseValue then
   _LabLimit := ((aFullGameInfo^[aPlayer].Base-UnlimitLabsBaseValue) div UnlimitLabsStep) + BaseLabLimit
                                                       else
   _LabLimit := BaseLabLimit;
  if aFullGameInfo^[aPlayer].Labs>_LabLimit then
   aFullGameInfo^[aPlayer].Labs:=_LabLimit;
end;

function DoProject(aFullGameInfo:PFullGameInfo; const CurrentPlayer:integer; aProject: integer):boolean;
begin
  Result:=IsProjectAvail(aFullGameInfo,CurrentPlayer,aProject);
  if Result then
   begin
    inc(aFullGameInfo^[CurrentPlayer].Statistic.ProjectsStat[aProject]);
    ProjectsHandlers[aProject](aFullGameInfo,CurrentPlayer);
    CheckLabLimit(aFullGameInfo,GetOtherPlayer(CurrentPlayer));
    CheckLabLimit(aFullGameInfo,CurrentPlayer);                
   end;
end;

end.

