unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FortressCommonDefs, FortressCore, StdCtrls, ExtCtrls, Spin, IntList;

const
  ProgramName = 'Fortress 2 build 2068';

type
  TBot = record
    Available:boolean;
    Name:string;
    StartGameFunc:TStartGameFunc;
    GetTurnFunc:TPlayerTurnFunc;
   end;

  TFortressMainForm = class(TForm)
    StartGameButton: TButton;
    AvailBotsComboBox: TComboBox;
    AvailSecondBotsComboBox: TComboBox;
    FirstPlayerGroupBox: TGroupBox;
    SecondPlayerGroupBox: TGroupBox;
    HumanCheckBox1: TCheckBox;
    HumanCheckBox2: TCheckBox;
    AboutButton: TButton;
    BotGameOptionsRadioGroup: TRadioGroup;
    GameCountSpinEdit: TSpinEdit;
    Label1: TLabel;
    AdditionalOptionsGroupBox: TGroupBox;
    Label2: TLabel;
    TurnLimitSpinEdit: TSpinEdit;
    procedure StartGameButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure HumanCheckBox1Click(Sender: TObject);
    procedure HumanCheckBox2Click(Sender: TObject);
    procedure AvailBotsComboBoxChange(Sender: TObject);
    procedure AboutButtonClick(Sender: TObject);
    procedure BotGameOptionsRadioGroupClick(Sender: TObject);
    procedure AvailSecondBotsComboBoxChange(Sender: TObject);
  private
    Bots:array of TBot;
    procedure UpdateBots;
    procedure UpdateStartGameButton;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FortressMainForm: TFortressMainForm;

implementation

{$R *.dfm}

uses GameResultUnit, Math;

function StartGame(aPlayerNumber:integer; aAvailProjects: PAvailProjects;
  AGI:PAdditionalGameInfo):integer; stdcall; external 'FortUI.dll';

function GetTurn(aPlayerNumber:integer; aGame:PGame; aAvailProjects: PAvailProjects;
  AGI:PAdditionalGameInfo):integer; stdcall; external 'FortUI.dll';

procedure ShowGameInterface; stdcall; external 'FortUI.dll';
procedure HideGameInterface; stdcall; external 'FortUI.dll';
procedure SetPlayerNames(P1Name,P2Name:string); stdcall; external 'FortUI.dll';
function ShowGame(aGame:PGame; Flags:DWORD; AGI:PAdditionalGameInfo):DWORD; stdcall; external 'FortUI.dll';

procedure TFortressMainForm.UpdateBots;
var
  GetBotNameFunc: TGetBotNameFunc;
  Func:Pointer;
  DllHandle:THandle;
  FindHandle : THandle;
  FindData : TWin32FindData;
  _i:integer;
  s: string;
begin
  AvailBotsComboBox.Clear;

  FindData.dwFileAttributes := FILE_ATTRIBUTE_NORMAL;
  FindHandle := FindFirstFile('*.dll', FindData);

  if FindHandle <> INVALID_HANDLE_VALUE then
   repeat
    s := FindData.cFileName;
    if (s<>'..') and (s<>'.') then
     AvailBotsComboBox.Items.Add(s);
   until not FindNextFile(FindHandle, FindData);
  windows.FindClose(FindHandle);

  SetLength(Bots,AvailBotsComboBox.Items.Count);
  for _i:=0 to AvailBotsComboBox.Items.Count-1 do
   begin
    DllHandle:=LoadLibrary(PChar(AvailBotsComboBox.Items[_i]));
    if DllHandle=0 then
     begin
      AvailBotsComboBox.Items[_i]:='';
      Bots[_i].Available:=false;
      Continue;
     end;
    Func:=GetProcAddress(DllHandle,'GetName');
    if Func=nil then
     begin
      AvailBotsComboBox.Items[_i]:='';
      Bots[_i].Available:=false;
      Continue;
     end;
     
    GetBotNameFunc:=TGetBotNameFunc(Func);   
    if not CheckBotName(GetBotNameFunc) then
     begin
      AvailBotsComboBox.Items[_i]:='';
      Bots[_i].Available:=false;
      Continue;
     end;
    Bots[_i].Name:=GetBotNameFunc;
    AvailBotsComboBox.Items[_i]:=Bots[_i].Name;

    Func:=GetProcAddress(DllHandle,'StartGame');
    if Func=nil then
     begin
      AvailBotsComboBox.Items[_i]:='';
      Bots[_i].Available:=false;
      Continue;
     end;
    Bots[_i].StartGameFunc:=TStartGameFunc(Func);

    Func:=GetProcAddress(DllHandle,'GetTurn');
    if Func=nil then
     begin
      AvailBotsComboBox.Items[_i]:='';
      Bots[_i].Available:=false;
      Continue;
     end;
    Bots[_i].GetTurnFunc:=TPlayerTurnFunc(Func);
    Bots[_i].Available:=true;
   end;

  AvailBotsComboBox.ItemIndex:=0;
  AvailBotsComboBox.Text:=AvailBotsComboBox.Items[AvailBotsComboBox.ItemIndex];

  AvailSecondBotsComboBox.Items.Clear;
  AvailSecondBotsComboBox.Items.AddStrings(AvailBotsComboBox.Items);
  AvailSecondBotsComboBox.ItemIndex:=0;
  AvailSecondBotsComboBox.Text:=AvailSecondBotsComboBox.Items[AvailSecondBotsComboBox.ItemIndex];
end;


type
  TGameResult = record
    RES:DWORD;
    GameInfo:TFullGameInfo;
    AGI:TAdditionalGameInfo;
    GameLog:TIntList;
   end;

  TGameOptions = record
    Options:DWORD;
    FirstPlayerName:string;
    SecondPlayerName:string;
   end;

var
  PerformanceFrequency:int64;

function GetTickCountEx:Double;
var
  res:int64;
begin
  QueryPerformanceCounter(res);
  Result:=(res/PerformanceFrequency)*1000;
end;

function GetScore(PlayerInfo:TFullPlayerInfo;TurnCount:DWORD):DWORD;
begin
  if TurnCount<>0 then
   Result:=round((TurnCount*MaxTurnTime-PlayerInfo.Statistic.Time)/(TurnCount*1000))
                  else
   Result:=2;               
end;

function Game(FirstPlayerStart,SecondPlayerStart: TStartGameFunc; FirstPlayerTurn,
  SecondPlayerturn: TPlayerTurnFunc; aGameOptions:TGameOptions):TGameResult;
label _END_GAME;
var
  _i:integer;
  FullGameInfo: TFullGameInfo;
  _Game: TGame;
  GameLog:TIntList;
  _AvailProjects : TAvailProjects;
  _Turn,_Count:integer;
  _DebugRes, _PlayerTurns, _temp:DWORD;
  _AGI:TAdditionalGameInfo;
  _time:Double;
begin
  QueryPerformanceFrequency(PerformanceFrequency);
  InitFullGameInfo(@FullGameInfo); 
  for _i:=0 to 1 do
   begin
    FullGameInfo[_i].Base:=40;
    FullGameInfo[_i].Shield:=15;

    FullGameInfo[_i].Battery:=3;
    FullGameInfo[_i].Mines:=2;
    FullGameInfo[_i].Labs:=2;

    FullGameInfo[_i].Energy:=15;
    FullGameInfo[_i].Metal:=15;
    FullGameInfo[_i].Elements:=15;
   end;
  ZeroMemory(@Result.AGI,sizeof(TAdditionalGameInfo));
  Result.AGI.GameFlags:=aGameOptions.Options;
  Result.GameInfo[0].Statistic.Time:=0;
  Result.GameInfo[1].Statistic.Time:=0;

  if (not IsFlagPresented(aGameOptions.Options,GF_BOT_vs_BOT)) or
     (IsFlagPresented(aGameOptions.Options,GF_DEBUGMODE)) then
   begin
    ShowGameInterface;
    SetPlayerNames(aGameOptions.FirstPlayerName,aGameOptions.SecondPlayerName);
   end;

  ZeroMemory(@_AvailProjects,sizeof(_AvailProjects));
  CopyMemory(@_AGI,@Result.AGI,sizeof(TAdditionalGameInfo));
  SetOppName(@_AGI,aGameOptions.SecondPlayerName);

  _time:=GetTickCountEx;
  FirstPlayerStart(0,@_AvailProjects,@_AGI);
  Result.GameInfo[0].Statistic.Time:=GetTickCountEx-_time;

  if not CheckAvailProjects(@_AvailProjects) then
   begin
    Result.RES:=GR_FirstLose or GR_ERROR;
    goto _END_GAME;
   end;
  CopyMemory(@FullGameInfo[0].Projects,@_AvailProjects,sizeof(_AvailProjects));

  ZeroMemory(@_AvailProjects,sizeof(_AvailProjects));
  CopyMemory(@_AGI,@Result.AGI,sizeof(TAdditionalGameInfo));
  SetOppName(@_AGI,aGameOptions.FirstPlayerName);

  _time:=GetTickCountEx;
  SecondPlayerStart(1,@_AvailProjects,@_AGI);
  Result.GameInfo[1].Statistic.Time:=GetTickCountEx-_time;

  if not CheckAvailProjects(@_AvailProjects) then
   begin
    Result.RES:=GR_SecondLose or GR_ERROR;
    goto _END_GAME;
   end;
  CopyMemory(@FullGameInfo[1].Projects,@_AvailProjects,sizeof(_AvailProjects));

  Result.GameLog:=TIntList.Create;
   
  repeat
   Result.AGI.TurnCount:=Result.AGI.TurnCount+1; 

   _Game:=FullGameInfoToGame(@FullGameInfo,Result.AGI.GameFlags, 0);
   CopyMemory(@_AGI,@Result.AGI,sizeof(TAdditionalGameInfo));
   SetOppName(@_AGI,aGameOptions.SecondPlayerName);
   
   _Count:=GetAvailProjects(@FullGameInfo[0],@_AvailProjects);
   if _Count<>0 then
    begin
     if (FullGameInfo[1].Shield>BlockShieldValue) then
      _AGI.PreviousOpponentProject:=0;

     _time:=GetTickCountEx;
     _Turn:=FirstPlayerTurn(0,@_Game,@_AvailProjects,@_AGI);
     _time:=GetTickCountEx-_time;
     FullGameInfo[0].Statistic.Time:=FullGameInfo[0].Statistic.Time+_time;

     Result.AGI.PreviousOpponentProject:=_Turn;
     if not DoProject(@FullGameInfo,0,_Turn) then
      begin
       Result.RES:=GR_FirstLose or GR_ERROR;
       break;
      end;
    end         else
    begin
     _time:=GetTickCountEx;
     FirstPlayerTurn(0,@_Game,0,@_AGI);
     _time:=GetTickCountEx-_time;
     FullGameInfo[0].Statistic.Time:=FullGameInfo[0].Statistic.Time+_time;
     Result.AGI.PreviousOpponentProject:=0;
    end;
   _PlayerTurns:=Result.AGI.PreviousOpponentProject;
   Result.GameLog.AddValue(Result.AGI.PreviousOpponentProject);

   if (_time>MaxTurnTime) and
      (IsFlagPresented(aGameOptions.Options,GF_BOT_vs_BOT)) then
    begin
     Result.RES:=GR_FirstLose or GR_TimeOut;
     break;
    end;
   if IsPlayerLose(@FullGameInfo[0]) then
    begin
     Result.RES:=GR_FirstLose;
     break;
    end;
   if IsPlayerLose(@FullGameInfo[1]) then
    begin
     Result.RES:=GR_SecondLose;
     break;
    end;    

   _Game:=FullGameInfoToGame(@FullGameInfo,Result.AGI.GameFlags,1);
   CopyMemory(@_AGI,@Result.AGI,sizeof(TAdditionalGameInfo));
   SetOppName(@_AGI,aGameOptions.FirstPlayerName);
   _Count:=GetAvailProjects(@FullGameInfo[1],@_AvailProjects);
   if _Count<>0 then
    begin
     if (FullGameInfo[0].Shield>BlockShieldValue) then
      _AGI.PreviousOpponentProject:=0;

     _time:=GetTickCountEx;
     _Turn:=SecondPlayerTurn(1,@_Game,@_AvailProjects,@_AGI);
     _time:=GetTickCountEx-_time;
     FullGameInfo[1].Statistic.Time:=FullGameInfo[1].Statistic.Time+_time;

     Result.AGI.PreviousOpponentProject:=_Turn;
     if not DoProject(@FullGameInfo,1,_Turn) then
      begin
       Result.RES:=GR_SecondLose or GR_ERROR;
       break;
      end;
    end         else
    begin
     _time:=GetTickCountEx;
     SecondPlayerTurn(1,@_Game,0,@_AGI);
     _time:=GetTickCountEx-_time;
     FullGameInfo[1].Statistic.Time:=FullGameInfo[1].Statistic.Time+_time;
     Result.AGI.PreviousOpponentProject:=0;     
    end;
   _PlayerTurns:=_PlayerTurns or (Result.AGI.PreviousOpponentProject shl 16);
   Result.GameLog.AddValue(Result.AGI.PreviousOpponentProject);
   
   if (_time>MaxTurnTime) and
      (IsFlagPresented(aGameOptions.Options,GF_BOT_vs_BOT)) then
    begin
     Result.RES:=GR_SecondLose or GR_TimeOut;
     break;
    end;
   if IsPlayerLose(@FullGameInfo[0]) then
    begin
     Result.RES:=GR_FirstLose;
     break;
    end;
   if IsPlayerLose(@FullGameInfo[1]) then
    begin
     Result.RES:=GR_SecondLose;
     break;
    end;        

   for _i:=0 to 1 do
    begin
     FullGameInfo[_i].Energy:=FullGameInfo[_i].Energy+FullGameInfo[_i].Battery;
     FullGameInfo[_i].Metal:=FullGameInfo[_i].Metal+FullGameInfo[_i].Mines;
     FullGameInfo[_i].Elements:=FullGameInfo[_i].Elements+FullGameInfo[_i].Labs;
     //update statistic
     FullGameInfo[_i].Statistic.TotalEnergyCollected:=FullGameInfo[_i].Statistic.TotalEnergyCollected+FullGameInfo[_i].Battery;
     FullGameInfo[_i].Statistic.TotalMetalCollected:=FullGameInfo[_i].Statistic.TotalMetalCollected+FullGameInfo[_i].Mines;
     FullGameInfo[_i].Statistic.TotalElementsCollected:=FullGameInfo[_i].Statistic.TotalElementsCollected+FullGameInfo[_i].Labs;
     FullGameInfo[_i].Statistic.BaseAvg:=FullGameInfo[_i].Statistic.BaseAvg+FullGameInfo[_i].Base;
     FullGameInfo[_i].Statistic.ShieldAvg:=FullGameInfo[_i].Statistic.ShieldAvg+FullGameInfo[_i].Shield;

     FullGameInfo[_i].Statistic.BatteriesAvg:=FullGameInfo[_i].Statistic.BatteriesAvg+FullGameInfo[_i].Battery;
     FullGameInfo[_i].Statistic.MinesAvg:=FullGameInfo[_i].Statistic.MinesAvg+FullGameInfo[_i].Mines;
     FullGameInfo[_i].Statistic.LabsAvg:=FullGameInfo[_i].Statistic.LabsAvg+FullGameInfo[_i].Labs;
    end;
      // 
   if IsFlagPresented(Result.AGI.GameFlags,GF_DEBUGMODE) then
    begin
     _temp:=Result.AGI.PreviousOpponentProject;
     _Game:=FullGameInfoToGame(@FullGameInfo, 0, 0);
     Result.AGI.PreviousOpponentProject:=_PlayerTurns;
     _DebugRes:=ShowGame(@_Game, 0, @Result.AGI);
     if _DebugRes=1 then //      
      begin
       aGameOptions.Options:=aGameOptions.Options and (not GF_DEBUGMODE);
       aGameOptions.Options:=aGameOptions.Options or GF_SINGLEGAME;
       Result.AGI.GameFlags:=aGameOptions.Options;
      end;
     if _DebugRes=2 then //    
      begin
        Result.RES:=GR_Draw;
        break;
      end;
     Result.AGI.PreviousOpponentProject:=_temp;
    end;
   if Result.AGI.TurnCount=(aGameOptions.Options and $FFFF) then
    begin
     Result.RES:=GR_Draw;
     break;
    end;
  until false;
_END_GAME:

  CopyMemory(@Result.GameInfo,@FullGameInfo,sizeof(TFullGameInfo));
  //AGI.GameResult !!!
  CopyMemory(@_AGI,@Result.AGI,sizeof(TAdditionalGameInfo));
  _AGI.GameResult:=Result.RES;
  if IsFlagPresented(Result.RES,GR_SecondLose) then
   _AGI.GameResult:=_AGI.GameResult or GR_WIN;
  SetOppName(@_AGI,aGameOptions.SecondPlayerName);
  FirstPlayerStart(0,0,@_AGI);

  CopyMemory(@_AGI,@Result.AGI,sizeof(TAdditionalGameInfo));
  _AGI.GameResult:=Result.RES;
  if IsFlagPresented(Result.RES,GR_FirstLose) then
   _AGI.GameResult:=_AGI.GameResult or GR_WIN;
  SetOppName(@_AGI,aGameOptions.FirstPlayerName);
  SecondPlayerStart(1,0,@_AGI);
  //  
  if Result.RES<>GR_Draw then
   begin
    CalcAvgs(Result.GameInfo[0].Statistic,Result.AGI.TurnCount-1);
    CalcAvgs(Result.GameInfo[1].Statistic,Result.AGI.TurnCount-1);
   end                   else
   begin
    CalcAvgs(Result.GameInfo[0].Statistic,Result.AGI.TurnCount);
    CalcAvgs(Result.GameInfo[1].Statistic,Result.AGI.TurnCount);
   end;
  if IsFlagPresented(aGameOptions.Options,GF_SINGLEGAME) then
   begin
    SetPlayerNames(aGameOptions.FirstPlayerName,aGameOptions.SecondPlayerName);
    ShowGameInterface;
    _Game:=FullGameInfoToGame(@FullGameInfo,0,0);
    ShowGame(@_GAME,0,@Result.AGI);
   end;
  HideGameInterface;
end;

function GenerateSpaces(val:integer):string;
var
  _i:integer;
begin
  Result:='';
  for _i:=1 to val do
   Result:=Result+' ';
end;

procedure WriteLogToMemo(Memo:TMemo; Log:TIntList);
var
  _i,_j:integer;
begin
  Memo.Lines.Add('-------------------------------------------');
  if not Assigned(Log) then exit;
  for _i:=0 to (Log.Count div 2)-1 do
   begin
    if Log.GetValue(_i*2)<>0 then
     Memo.Lines.Add(inttostr(_i+1)+'.1 '+ProjectsStrings[Log.GetValue(_i*2)])
       else
     Memo.Lines.Add(inttostr(_i+1)+'.1  ');
    if Log.GetValue(_i*2+1)<>0 then
     Memo.Lines.Add(inttostr(_i+1)+'.2 '+ProjectsStrings[Log.GetValue(_i*2+1)])
       else
     Memo.Lines.Add(inttostr(_i+1)+'.2  ');
   end;
  if (Log.Count mod 2) <>0 then
   if Log.GetValue(Log.Count-1)<>0 then
    Memo.Lines.Add(inttostr(_i+1)+'.1 '+ProjectsStrings[Log.GetValue(Log.Count-1)])
                                   else
    Memo.Lines.Add(inttostr(_i+1)+'.1  ');
end;

procedure WriteStatToMemo(Memo:TMemo; aPlayerStat:TPlayerStat);
var
  _i,_TotalProjects:integer;
  _PR:Double;
  _str,_pr_str:string;
begin
  Memo.Lines.Add('-------------------------------------------');
  Memo.Lines.Add('  : '+FloatToStr(roundTo(aPlayerStat.Time,-3))+' ');
  Memo.Lines.Add('   : '+FloatToStrF(aPlayerStat.BaseAvg, ffGeneral, 3, 0));
  Memo.Lines.Add('   : '+FloatToStrF(aPlayerStat.ShieldAvg, ffGeneral, 3, 0));

  Memo.Lines.Add('   : '+FloatToStrF(aPlayerStat.BatteriesAvg, ffGeneral, 3, 0));
  Memo.Lines.Add('   : '+FloatToStrF(aPlayerStat.MinesAvg, ffGeneral, 3, 0));
  Memo.Lines.Add('   : '+FloatToStrF(aPlayerStat.LabsAvg, ffGeneral, 3, 0));
  Memo.Lines.Add('   : '+IntToStr(aPlayerStat.TotalEnergyCollected));
  Memo.Lines.Add('   : '+IntToStr(aPlayerStat.TotalMetalCollected));
  Memo.Lines.Add('   : '+IntToStr(aPlayerStat.TotalElementsCollected));
  Memo.Lines.Add(' ');
  _TotalProjects:=0;
  for _i:=1 to TotalProjectsCount do
   _TotalProjects:=_TotalProjects+aPlayerStat.ProjectsStat[_i];
                                                                 
  Memo.Lines.Add('|     %|');
  for _i:=1 to TotalProjectsCount do
   begin
    _str:=IntToStr(aPlayerStat.ProjectsStat[_i]);
    _str:=GenerateSpaces(9-length(_str))+_str+' |';
    if _TotalProjects<>0 then
     _PR:= (aPlayerStat.ProjectsStat[_i]/_TotalProjects)*100
                         else
     _PR:=0;
    _pr_str:=FloatToStr(roundTo(_PR,-2));
    _str:=_str+GenerateSpaces(5-length(_pr_str))+_pr_str+'%|';
    _str:=_str+  ProjectsStrings[_i];
    Memo.Lines.Add(_str);
   end;
  Memo.Lines.Add('-------------------------------------------');
end;

procedure TFortressMainForm.StartGameButtonClick(Sender: TObject);
var
  RES:TGameResult;
  GameOptions:TGameOptions;
  str:string;
  i:integer;
  StartGameP1,StartGameP2,tempSG:TStartGameFunc;
  GetTurnP1,GetTurnP2,tempGT:TPlayerTurnFunc;

  P1WinCount,P2WinCount:integer;
  P1ErrorCount,P2ErrorCount:integer;
  P1Score, P2Score, TotalGameCount: DWORD;
  P1TotalScore, P2TotalScore: DWORD;
  P1TotalStat, P2TotalStat, tempStat: TPlayerStat;
  DrawCount,temp:integer;
begin
  Hide;
  if HumanCheckBox1.Checked then
   begin
    StartGameP1:=StartGame;
    GetTurnP1:=GetTurn;
    GameOptions.FirstPlayerName:=HUMAN_NAME;
   end                      else
   begin
    StartGameP1:=Bots[AvailBotsComboBox.ItemIndex].StartGameFunc;
    GetTurnP1:=Bots[AvailBotsComboBox.ItemIndex].GetTurnFunc;
    GameOptions.FirstPlayerName:=Bots[AvailBotsComboBox.ItemIndex].Name;
   end;

  if HumanCheckBox2.Checked then
   begin
    StartGameP2:=StartGame;
    GetTurnP2:=GetTurn;
    GameOptions.SecondPlayerName:=HUMAN_NAME;
   end                      else
   begin
    StartGameP2:=Bots[AvailSecondBotsComboBox.ItemIndex].StartGameFunc;
    GetTurnP2:=Bots[AvailSecondBotsComboBox.ItemIndex].GetTurnFunc;
    GameOptions.SecondPlayerName:=Bots[AvailSecondBotsComboBox.ItemIndex].Name;
   end;

  GameOptions.Options:=0;
  if not(HumanCheckBox1.Checked or HumanCheckBox2.Checked) then
   GameOptions.Options:=GF_BOT_vs_BOT;
  if IsFlagPresented(GameOptions.Options,GF_BOT_vs_BOT) then
   begin
    if BotGameOptionsRadioGroup.ItemIndex=0 then
     GameOptions.Options:=GameOptions.Options or GF_SHOWGAMERESULT;
    if BotGameOptionsRadioGroup.ItemIndex=1 then
     GameOptions.Options:=GameOptions.Options or GF_DEBUGMODE;
   end;
  GameOptions.Options:=GameOptions.Options or (GF_EXTENDED or DWORD(TurnLimitSpinEdit.Value));

  if (HumanCheckBox1.Checked) or
     (HumanCheckBox2.Checked) or
     (
      (not HumanCheckBox1.Checked) and
      (not HumanCheckBox2.Checked) and
      (
       (BotGameOptionsRadioGroup.ItemIndex=0) or
       (BotGameOptionsRadioGroup.ItemIndex=1)
       )
      ) then
   begin
    RES:=Game(StartGameP1,StartGameP2,GetTurnP1,GetTurnP2,GameOptions);

    GameResultForm.ResultMemo.Clear;
    GameResultForm.ResultMemo.Lines.Add('  : '+GameOptions.FirstPlayerName);
    GameResultForm.ResultMemo.Lines.Add('  : '+GameOptions.SecondPlayerName);
    GameResultForm.ResultMemo.Lines.Add('  : '+IntToStr(res.AGI.TurnCount));
    str:='';
    if IsFlagPresented(RES.RES,GR_FirstLose) then
     str:='   ('+GameOptions.SecondPlayerName+')';
    if IsFlagPresented(RES.RES,GR_SecondLose) then
     str:='   ('+GameOptions.FirstPlayerName+')';
    if IsFlagPresented(RES.RES,GR_Draw) then
     str:='';
    if IsFlagPresented(RES.RES,GR_ERROR) then
     str:=str+' (   )';
    if IsFlagPresented(RES.RES,GR_TimeOut) then
     str:=str+' (   )';
    GameResultForm.ResultMemo.Lines.Add(str);
    P1Score:=GetScore(RES.GameInfo[0],RES.AGI.TurnCount);
    P2Score:=GetScore(RES.GameInfo[1],RES.AGI.TurnCount);
    if IsFlagPresented(RES.RES,GR_FirstLose) then P2Score:=P2Score*10;
    if IsFlagPresented(RES.RES,GR_SecondLose) then P1Score:=P1Score*10;
    GameResultForm.ResultMemo.Lines.Add('   : '+IntToStr(P1Score));
    GameResultForm.ResultMemo.Lines.Add('   : '+IntToStr(P2Score));
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('   ('+GameOptions.FirstPlayerName+')');
    WriteStatToMemo(GameResultForm.ResultMemo,RES.GameInfo[0].Statistic);
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('   ('+GameOptions.SecondPlayerName+')');
    WriteStatToMemo(GameResultForm.ResultMemo,RES.GameInfo[1].Statistic);
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add(' ');
    WriteLogToMemo(GameResultForm.ResultMemo,RES.GameLog);
    //RES.GameLog.Free;
    GameResultForm.ShowModal;
   end  else
   begin
    P1WinCount:=0;
    P2WinCount:=0;
    DrawCount:=0;
    P1ErrorCount:=0;
    P2ErrorCount:=0;
    P1TotalScore:=0;
    P2TotalScore:=0;
    ZeroMemory(@P1TotalStat,sizeof(P1TotalStat));
    ZeroMemory(@P2TotalStat,sizeof(P2TotalStat));
    for i:=1 to  GameCountSpinEdit.Value do
     begin
      RES:=Game(StartGameP1,StartGameP2,GetTurnP1,GetTurnP2,GameOptions);
      RES.GameLog.Free;
      if IsFlagPresented(RES.RES,GR_FirstLose) then  inc(P2WinCount);
      if IsFlagPresented(RES.RES,GR_SecondLose) then  inc(P1WinCount);
      if IsFlagPresented(RES.RES,GR_Draw) then inc(DrawCount);
      if IsFlagPresented(RES.RES,GR_FirstLose) and
         IsFlagPresented(RES.RES,GR_ERROR) then inc(P1ErrorCount);
      if IsFlagPresented(RES.RES,GR_SecondLose) and
         IsFlagPresented(RES.RES,GR_ERROR) then inc(P2ErrorCount);
      P1Score:=GetScore(RES.GameInfo[0],RES.AGI.TurnCount);
      P2Score:=GetScore(RES.GameInfo[1],RES.AGI.TurnCount);
      if IsFlagPresented(RES.RES,GR_FirstLose) then P2Score:=P2Score*10;
      if IsFlagPresented(RES.RES,GR_SecondLose) then P1Score:=P1Score*10;
      P1TotalScore:=P1TotalScore+P1Score;
      P2TotalScore:=P2TotalScore+P2Score;
      AddStat(P1TotalStat,RES.GameInfo[0].Statistic);
      AddStat(P2TotalStat,RES.GameInfo[1].Statistic);
     end;
    TotalGameCount:=GameCountSpinEdit.Value;
    if BotGameOptionsRadioGroup.ItemIndex=3 then
     begin
      //  
      tempSG:=StartGameP1;
      StartGameP1:=StartGameP2;
      StartGameP2:=tempSG;
      tempGT:=GetTurnP1;
      GetTurnP1:=GetTurnP2;
      GetTurnP2:=tempGT;
      // 
      str:=GameOptions.FirstPlayerName;
      GameOptions.FirstPlayerName:=GameOptions.SecondPlayerName;
      GameOptions.SecondPlayerName:=str;
   
      //   
      temp:=P1WinCount;
      P1WinCount:=P2WinCount;
      P2WinCount:=temp;
      temp:=P1ErrorCount;
      P1ErrorCount:=P2ErrorCount;
      P2ErrorCount:=temp;
      temp:=P1TotalScore;
      P1TotalScore:=P2TotalScore;
      P2TotalScore:=temp;
      CopyMemory(@tempStat,@P1TotalStat,sizeof(TPlayerStat));
      CopyMemory(@P1TotalStat,@P2TotalStat,sizeof(TPlayerStat));
      CopyMemory(@P2TotalStat,@tempStat,sizeof(TPlayerStat));
   
      for i:=1 to  GameCountSpinEdit.Value do
       begin
        RES:=Game(StartGameP1,StartGameP2,GetTurnP1,GetTurnP2,GameOptions);
        RES.GameLog.Free;
        if IsFlagPresented(RES.RES,GR_FirstLose) then  inc(P2WinCount);
        if IsFlagPresented(RES.RES,GR_SecondLose) then  inc(P1WinCount);
        if IsFlagPresented(RES.RES,GR_Draw) then inc(DrawCount);
        if IsFlagPresented(RES.RES,GR_FirstLose) and
           IsFlagPresented(RES.RES,GR_ERROR) then inc(P1ErrorCount);
        if IsFlagPresented(RES.RES,GR_SecondLose) and
           IsFlagPresented(RES.RES,GR_ERROR) then inc(P2ErrorCount);
        P1Score:=GetScore(RES.GameInfo[0],RES.AGI.TurnCount);
        P2Score:=GetScore(RES.GameInfo[1],RES.AGI.TurnCount);
        if IsFlagPresented(RES.RES,GR_FirstLose) then P2Score:=P2Score*10;
        if IsFlagPresented(RES.RES,GR_SecondLose) then P1Score:=P1Score*10;
        P1TotalScore:=P1TotalScore+P1Score;
        P2TotalScore:=P2TotalScore+P2Score;
        AddStat(P1TotalStat,RES.GameInfo[0].Statistic);
        AddStat(P2TotalStat,RES.GameInfo[1].Statistic);
       end;
      TotalGameCount:=TotalGameCount+GameCountSpinEdit.Value; 
      //   (   )
      temp:=P1WinCount;
      P1WinCount:=P2WinCount;
      P2WinCount:=temp;
      temp:=P1ErrorCount;
      P1ErrorCount:=P2ErrorCount;
      P2ErrorCount:=temp;
      //     (   )
      str:=GameOptions.FirstPlayerName;
      GameOptions.FirstPlayerName:=GameOptions.SecondPlayerName;
      GameOptions.SecondPlayerName:=str;
   
      temp:=P1TotalScore;
      P1TotalScore:=P2TotalScore;
      P2TotalScore:=temp;
      CopyMemory(@tempStat,@P1TotalStat,sizeof(TPlayerStat));
      CopyMemory(@P1TotalStat,@P2TotalStat,sizeof(TPlayerStat));
      CopyMemory(@P2TotalStat,@tempStat,sizeof(TPlayerStat));
     end;
    CalcAvgs(P1TotalStat, TotalGameCount);
    CalcAvgs(P2TotalStat, TotalGameCount);
    GameResultForm.ResultMemo.Clear;
    GameResultForm.ResultMemo.Lines.Add('  : '+GameOptions.FirstPlayerName);
    GameResultForm.ResultMemo.Lines.Add('  : '+GameOptions.SecondPlayerName);
    GameResultForm.ResultMemo.Lines.Add(' : '+IntToStr(TotalGameCount));

    GameResultForm.ResultMemo.Lines.Add(GameOptions.FirstPlayerName+' :   '+IntToStr(P1WinCount));
    GameResultForm.ResultMemo.Lines.Add(GameOptions.SecondPlayerName+' :   '+IntToStr(P2WinCount));
    GameResultForm.ResultMemo.Lines.Add('  '+IntToStr(DrawCount));
    GameResultForm.ResultMemo.Lines.Add(GameOptions.FirstPlayerName+' :   '+IntToStr(P1ErrorCount));
    GameResultForm.ResultMemo.Lines.Add(GameOptions.SecondPlayerName+' :   '+IntToStr(P2ErrorCount));

    GameResultForm.ResultMemo.Lines.Add(GameOptions.FirstPlayerName+' : '+IntToStr(P1TotalScore)+' ');
    GameResultForm.ResultMemo.Lines.Add(GameOptions.SecondPlayerName+' : '+IntToStr(P2TotalScore)+' ');

    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('   ('+GameOptions.FirstPlayerName+')');
    WriteStatToMemo(GameResultForm.ResultMemo,P1TotalStat);
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('');
    GameResultForm.ResultMemo.Lines.Add('   ('+GameOptions.SecondPlayerName+')');
    WriteStatToMemo(GameResultForm.ResultMemo,P2TotalStat);
    GameResultForm.ShowModal;
   end;
  Show;
end;

procedure TFortressMainForm.FormActivate(Sender: TObject);
begin
  Application.Title:=ProgramName;
  Caption:=Application.Title;
  UpdateBots;
  AvailBotsComboBoxChange(AvailBotsComboBox);
  AvailSecondBotsComboBoxChange(AvailSecondBotsComboBox);
end;

procedure TFortressMainForm.HumanCheckBox1Click(Sender: TObject);
begin
  AvailBotsComboBox.Enabled:= not HumanCheckBox1.Checked;
  BotGameOptionsRadioGroup.Enabled:= not (HumanCheckBox1.Checked or HumanCheckBox2.Checked);
 // AdditionalOptionsGroupBox.Enabled:=BotGameOptionsRadioGroup.Enabled;
  if BotGameOptionsRadioGroup.Enabled then
   BotGameOptionsRadioGroupClick(BotGameOptionsRadioGroup);

  if HumanCheckBox1.Checked then
   begin
    Label1.Enabled:=false;
    GameCountSpinEdit.Enabled:=Label1.Enabled;
   end;
  UpdateStartGameButton; 
end;

procedure TFortressMainForm.HumanCheckBox2Click(Sender: TObject);
begin
  AvailSecondBotsComboBox.Enabled:= not HumanCheckBox2.Checked;
  BotGameOptionsRadioGroup.Enabled:= not (HumanCheckBox1.Checked or HumanCheckBox2.Checked);
  //AdditionalOptionsGroupBox.Enabled:=BotGameOptionsRadioGroup.Enabled;
  if BotGameOptionsRadioGroup.Enabled then
   BotGameOptionsRadioGroupClick(BotGameOptionsRadioGroup);

  if HumanCheckBox2.Checked then
   begin
    Label1.Enabled:=false;
    GameCountSpinEdit.Enabled:=Label1.Enabled;
   end;
  UpdateStartGameButton; 
end;

procedure TFortressMainForm.AvailBotsComboBoxChange(Sender: TObject);
begin
  UpdateStartGameButton;
end;

procedure TFortressMainForm.AboutButtonClick(Sender: TObject);
begin
  ShowMessage(ProgramName+#13#10': rpy3uH'#13#10'E-mail: gruzino.abhaz@gmail.com');
end;

procedure TFortressMainForm.BotGameOptionsRadioGroupClick(Sender: TObject);
begin
  Label1.Enabled:=(BotGameOptionsRadioGroup.ItemIndex=2)
              or  (BotGameOptionsRadioGroup.ItemIndex=3);
  GameCountSpinEdit.Enabled:=Label1.Enabled;
end;

procedure TFortressMainForm.UpdateStartGameButton;
begin
  if HumanCheckBox1.Checked and HumanCheckBox2.Checked then
   StartGameButton.Enabled:=true                       else
   begin
     if  HumanCheckBox1.Checked and (not HumanCheckBox2.Checked) then
      StartGameButton.Enabled:= Bots[AvailSecondBotsComboBox.ItemIndex].Available;
     if  HumanCheckBox2.Checked and (not HumanCheckBox1.Checked) then
      StartGameButton.Enabled:= Bots[AvailBotsComboBox.ItemIndex].Available;
     if (not HumanCheckBox2.Checked) and (not HumanCheckBox1.Checked) then
      StartGameButton.Enabled:= Bots[AvailBotsComboBox.ItemIndex].Available
                            and Bots[AvailSecondBotsComboBox.ItemIndex].Available;
   end;
end;

procedure TFortressMainForm.AvailSecondBotsComboBoxChange(Sender: TObject);
begin
  UpdateStartGameButton;
end;

end.
